import os
from PIL import Image
from PIL import ImageDraw
from PIL import ImageFont

from dummyimage.forms import DummyImageForm

# TODO: move to settings
FONT_FILE = 'DroidSans.ttf'
FONT_PATH = os.path.join(os.path.dirname(__file__), 'fonts', FONT_FILE)


class DummyImage(object):

    class InvalidParams(Exception):
        def __init__(self, message, form):
            self.form = form
            super(DummyImage.InvalidParams, self).__init__(message)

    @classmethod
    def new(cls, width, height, **kwargs):
        data = kwargs.copy()
        data.update({
            'width': width,
            'height': height,
        })
        form = DummyImageForm(data=data)

        if not form.is_valid():
            raise  cls.InvalidParams("Invalid image params", form)

        # custom text
        text = form.cleaned_data['text']

        # mode. Use RGBA if transparent
        mode = 'RGBA' if form.cleaned_data.get('transparent') else 'RGB'
        width = form.cleaned_data['width']
        height = form.cleaned_data['height']
        size = (width, height)

        # allow transparent color
        if 'RGBA' == mode:
            bgcolor = None
        else:
            bgcolor = form.cleaned_data['bgcolor']

        # color allows short hex format
        image = Image.new(mode, size, bgcolor)
        draw = ImageDraw.Draw(image)

        bordercolor = form.cleaned_data['bordercolor']
        # draw border
        if not form.cleaned_data.get('noborder'):
            draw.polygon([(0, 0), (width - 1, 0), (width - 1, height - 1),
                        (0, height - 1)], outline=bordercolor)

        # draw cross
        if form.cleaned_data.get('cross'):
            draw.line([(0, 0), (width - 1, height - 1)], fill=bordercolor)
            draw.line([(0, height - 1), (width - 1, 0)], fill=bordercolor)

        # draw text centered
        if text:
            font = ImageFont.truetype(FONT_PATH, width / 10)

            center = (width / 2, height / 2)
            text_size = font.getsize(text)
            text_center = (center[0] - text_size[0] / 2,
                        center[1] - text_size[1] / 2)
            draw.text(text_center, text, font=font,
                      fill=form.cleaned_data['textcolor'])

        return image



DEBUG = True
TEMPLATE_DEBUG = DEBUG

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    }
}

DUMMYIMAGE_MAX_DIMENSION = 1024

ROOT_URLCONF = 'dummyimage.urls'

INSTALLED_APPS = (
    'dummyimage',
)




from django.conf.urls.defaults import *
from django.http import HttpResponseNotFound

WIDTH_RE = r'(?P<width>\d+)'
HEIGHT_RE = r'(?P<height>\d+)'
FORMAT_RE = r'(?P<format>[a-z]{3})'
IMAGE_RE = r'^%sx%s\.%s$' % (WIDTH_RE, HEIGHT_RE, FORMAT_RE)

urlpatterns = patterns('dummyimage.views',
    (IMAGE_RE, 'render_image'),
)


def view404(request):
    return HttpResponseNotFound()


handler404 = view404



from django.core.urlresolvers import reverse
from django.template import Context
from django.template import Template

from django.test import Client
from django.test import TestCase

from dummyimage.forms import _get_color


class TemplateTagTest(TestCase):
    def setUp(self):
        self.client = Client()
        self.get_url = lambda w, h, fmt: reverse(
                            'dummyimage.views.render_image',
                            args=(w, h, fmt))

    def get_template(self, arguments):
        return '{%% load dummyimage_tags %%}{%% get_dummyimage_url %s %%}' % arguments

    def test_render_filename(self):
        url = self.get_url(100, 100, 'jpg')
        t = Template(self.get_template('100x100.jpg'))
        c = Context({})
        self.failUnlessEqual(url, t.render(c))

    def test_render_filename_to_context(self):
        url = self.get_url(100, 100, 'jpg')
        t = Template(self.get_template('100x100.jpg as myvar'))
        c = Context({})
        t.render(c)
        self.failUnlessEqual(url, c['myvar'])

    def test_render_width_and_height(self):
        url = self.get_url(100, 100, 'jpg')
        #@@@ default format jpg
        t = Template(self.get_template('100 100'))
        c = Context({})
        self.failUnlessEqual(url, t.render(c))

    def test_render_width_and_height_to_context(self):
        url = self.get_url(100, 100, 'jpg')
        #@@@ default format jpg
        t = Template(self.get_template('100 100 as myvar'))
        c = Context({})
        t.render(c)
        self.failUnlessEqual(url, c['myvar'])

    def test_render_width_height_format(self):
        url = self.get_url(100, 200, 'png')
        #@@@ default format jpg
        t = Template(self.get_template('100 200 png'))
        c = Context({})
        self.failUnlessEqual(url, t.render(c))

    def test_render_width_height_format_to_context(self):
        url = self.get_url(100, 200, 'png')
        #@@@ default format jpg
        t = Template(self.get_template('100 200 png as myvar'))
        c = Context({})
        t.render(c)
        self.failUnlessEqual(url, c['myvar'])


class RenderViewTest(TestCase):
    def setUp(self):
        self.client = Client()
        self.get_url = lambda w, h, fmt: reverse(
                            'dummyimage.views.render_image',
                            args=(w, h, fmt))

    def test_sizes_boundaries(self):
        # sizes
        response = self.client.get(self.get_url(0, 0, 'jpg'))
        self.failUnlessEqual(response.status_code, 404)

        response = self.client.get(self.get_url(1, 1, 'jpg'))
        self.failUnlessEqual(response.status_code, 200)

        response = self.client.get(self.get_url(1024, 1024, 'jpg'))
        self.failUnlessEqual(response.status_code, 200)

        response = self.client.get(self.get_url(1025, 1025, 'jpg'))
        self.failUnlessEqual(response.status_code, 404)

    def test_valid_formats(self):
        # formats
        response = self.client.get(self.get_url(1, 1, 'jpg'))
        self.failUnlessEqual(response.status_code, 200)
        self.failUnlessEqual(response['Content-Type'], 'image/jpeg')

        response = self.client.get(self.get_url(1, 1, 'gif'))
        self.failUnlessEqual(response.status_code, 200)
        self.failUnlessEqual(response['Content-Type'], 'image/gif')

        response = self.client.get(self.get_url(1, 1, 'png'))
        self.failUnlessEqual(response.status_code, 200)
        self.failUnlessEqual(response['Content-Type'], 'image/png')

    def test_invalid_formats(self):
        response = self.client.get(self.get_url(1, 1, 'avi'))
        self.failUnlessEqual(response.status_code, 404)

        response = self.client.get(self.get_url(1, 1, 'pic'))
        self.failUnlessEqual(response.status_code, 404)

        response = self.client.get(self.get_url(1, 1, 'bmp'))
        self.failUnlessEqual(response.status_code, 404)


    def test_rotation_param(self):
        ## Check Params
        url = self.get_url(1, 1, 'jpg')

        # rotation
        response = self.client.get(url, {'rotate': '-359'})
        self.failUnlessEqual(response.status_code, 200)
        response = self.client.get(url, {'rotate': '0'})
        self.failUnlessEqual(response.status_code, 200)
        response = self.client.get(url, {'rotate': '359'})
        self.failUnlessEqual(response.status_code, 200)

    def test_invalid_rotation_param(self):
        url = self.get_url(1, 1, 'jpg')
        # invalid
        response = self.client.get(url, {'rotate': 'asdf'})
        self.failUnlessEqual(response.status_code, 404)

        response = self.client.get(url, {'rotate': '-360'})
        self.failUnlessEqual(response.status_code, 404)
        response = self.client.get(url, {'rotate': '360'})
        self.failUnlessEqual(response.status_code, 404)

    def test_text_parameters(self):
        url = self.get_url(1, 1, 'jpg')

        response = self.client.get(url, {'text': 'asdf'})
        # TODO: verify actual font rendering
        self.assertEquals(response.status_code, 200)


class GetColorTest(TestCase):
    def test_get_color_util(self):
        """
        Tests return of _get_color function
        """
        #@@@ defaults: bg -> white, text -> black, border -> grey

    def test_default_returns(self):
        # Test defaults
        self.failUnlessEqual('white', _get_color('', 'BG'))
        self.failUnlessEqual('grey', _get_color('', 'TEXT'))
        self.failUnlessEqual('grey', _get_color('', 'BORDER'))

    def test_default_values_if_invalid(self):
        self.failUnlessEqual('white', _get_color('!invalid', 'BG'))
        self.failUnlessEqual('grey', _get_color('!invalid', 'TEXT'))
        self.failUnlessEqual('grey', _get_color('!invalid', 'BORDER'))

    def test_hex_values(self):
        # Test hex values
        self.failUnlessEqual('#000000', _get_color('!000000', 'BG'))
        self.failUnlessEqual('#FFFFFF', _get_color('!ffffff', 'BG'))

    def test_short_hex_values(self):
        # Shorts
        self.failUnlessEqual('#001122', _get_color('!012', 'BG'))
        self.failUnlessEqual('#AABBCC', _get_color('!abc', 'BG'))

    def test_invalid_hex_values(self):
        # invalids
        self.failUnlessEqual('grey', _get_color('!xyz', 'TEXT'))
        self.failUnlessEqual('grey', _get_color('!1234567', 'BORDER'))

    def test_invalid_type(self):
        # Test exceptions
        # invalid type
        self.failUnlessRaises(KeyError, _get_color, '', 'TYPE')
        self.failUnlessRaises(KeyError, _get_color, '!invalid', 'TYPE')




from django.http import Http404
from django.http import HttpResponse

from .models import DummyImage

AVAILABLE_FORMATS = ('jpeg', 'gif', 'png')


def render_image(request, width, height, format):
    params = dict(request.GET.items())
    image_format = 'jpeg' if format == 'jpg' else format
    try:
        dummyimage = DummyImage.new(width, height, **params)
    except DummyImage.InvalidParams:
        raise Http404

    if image_format not in AVAILABLE_FORMATS:
        raise Http404

    # write image to response
    response = HttpResponse(mimetype='image/%s' % image_format)
    #TODO: catch exceptions
    dummyimage.save(response, image_format)

    return response



from django.core.urlresolvers import reverse
from django.template import Library
from django.template import Node
from django.template import TemplateSyntaxError
from django.template import Variable
from django.template import resolve_variable
from django.utils.translation import ugettext as _

register = Library()

class DummyImageUrlNode(Node):
    def __init__(self, width, height, format, context_var):
        # TODO: support variable names instead of only integers
        self.width = width
        self.height = height
        self.format = format
        self.context_var = context_var

    def render(self, context):
        url = reverse('dummyimage.views.render_image',
                args=(self.width, self.height, self.format))

        if self.context_var:
            # push url to context
            context[self.context_var] = url
            return ''
        else:
            # render url
            return url

def do_get_dummyimage_url(parser, token):
    """
    Returns dummy image URL

    Available formats: gif, jpg, png
    Filename format: {w}x{h}.{format} e.g. 100x100.jpg

    Usage::
        {% get_dummyimage_url [width] [height] [format] as [varname] %}
        {% get_dummyimage_url [width] [height] as [varname] %}
        {% get_dummyimage_url [filename] as [varname] %}

        {% get_dummyimage_url [width] [height] [format] %}
        {% get_dummyimage_url [width] [height] %}
        {% get_dummyimage_url [filename] %}

    Example::
        <img src="{% get_dummyimage_url 320x240.png %}" />
        <img src="{% get_dummyimage_url 320 240 %}" />

        {% get_dummyimage_url 320 240 png as image_url %}
        {% get_dummyimage_url 320 240 as image_url %}
        {% get_dummyimage_url 320x240.gif as image_url %}
    """
    bits = token.contents.split()
    # optional arguments
    format = None
    varname = None

    # tag + 1-5 arguments
    if not 1 < len(bits) <= 6:
        raise TemplateSyntaxError(_("%s tag requires between one and five arguments") % bits[0])

    # tag + 5 arguments
    if len(bits) == 6:
        if not bits[4] == 'as':
            raise TemplateSyntaxError(_("if given, fourth argument to %s tag must be 'as'") % bits[0])

        width = bits[1]
        height = bits[2]
        format = bits[3]
        varname = bits[5]
 
    # tag + 4 arguments
    if len(bits) == 5:
        if not bits[3] == 'as':
            raise TemplateSyntaxError(_("if given, third argument to %s tag must be 'as'") % bits[0])

        width = bits[1]
        height = bits[2]
        varname = bits[4]

    # tag + 3 arguments
    if len(bits) == 4:
        if bits[2] == 'as':
            varname = bits[3]
            try:
                width, remain = bits[1].split('x')
                height, format = remain.split('.')
            except ValueError:
                raise TemplateSyntaxError(_("first argument to %s tag must be in format 'WidthxHeight.format'. e.g. 320x240.png") % bits[0])
        else:
            width = bits[1]
            height = bits[2]
            format = bits[3]

    # tag + 2 arguments
    if len(bits) == 3:
        width = bits[1]
        height = bits[2]

    # tag + 1 argument
    if len(bits) == 2:
        try:
            width, remain = bits[1].split('x')
            height, format = remain.split('.')
        except ValueError:
            raise TemplateSyntaxError(_("first argument to %s tag must be in format 'WidthxHeight.format'. e.g. 320x240.png") % bits[0])

    # validate width and height integers
    try:
        width = int(width)
        height = int(height)
    except ValueError:
        raise TemplateSyntaxError(_("width and height argument to %s tag must be integers") % bits[0])

    # validate format
    if not format:
        format = 'jpg' # default
    elif not format in ('jpg', 'gif', 'png'):
        raise TemplateSyntaxError(_("format argument must be either jpg, png or gif"))

    return DummyImageUrlNode(width, height, format, varname)

register.tag('get_dummyimage_url', do_get_dummyimage_url)





import re
import sys
from django import forms
from django.conf import settings


MAX_DIMENSION = getattr(settings, 'DUMMYIMAGE_MAX_DIMENSION', sys.maxint)
DEFAULT_BG = getattr(settings, 'DUMMYIMAGE_DEFAULT_BG', 'white')
DEFAULT_TEXT = getattr(settings, 'DUMMYIMAGE_DEFAULT_TEXT', 'grey')
DEFAULT_BORDER = getattr(settings, 'DUMMYIMAGE_DEFAULT_BORDER', 'grey')

DEFAULT_COLORS = {
    'BG': DEFAULT_BG,
    'TEXT': DEFAULT_TEXT,
    'BORDER': DEFAULT_BORDER,
    }

RE_HEX_FULL = re.compile('^[A-F0-9]{6}$', re.IGNORECASE)
RE_HEX_SHORT = re.compile('^[A-F0-9]{3}$', re.IGNORECASE)


def _get_color(value, type):
    """Returns valid color"""
    if value and value.startswith('!'):
        # normalize hex color
        if RE_HEX_FULL.match(value[1:]):
            color = '#%s' % ''.join(value[1:]).upper()
        elif RE_HEX_SHORT.match(value[1:]):
            # duplicate short values
            color = '#%s' % ''.join(c * 2 for c in value[1:]).upper()
        else:
            # invalid color
            color = DEFAULT_COLORS[type]
        return color
    else:
        # assume literal color. e.g. white, grey69, etc
        return value if value else DEFAULT_COLORS[type]


class DummyImageForm(forms.Form):
    bgcolor = forms.CharField(initial=DEFAULT_BG, required=False)
    transparent = forms.BooleanField(initial=False, required=False)

    text = forms.CharField(required=False)
    textcolor = forms.CharField(initial=DEFAULT_TEXT, required=False)

    bordercolor = forms.CharField(initial=DEFAULT_BORDER, required=False)
    noborder = forms.BooleanField(initial=False, required=False)
    cross = forms.BooleanField(initial=False, required=False)

    width = forms.IntegerField(min_value=1, max_value=MAX_DIMENSION)
    height = forms.IntegerField(min_value=1, max_value=MAX_DIMENSION)
    rotate = forms.IntegerField(min_value=-359, max_value=359, initial=0,
                                required=False)

    def clean_bgcolor(self):
        color = self.cleaned_data['bgcolor']
        color = _get_color(color, 'BG')
        return color

    def clean_textcolor(self):
        color = self.cleaned_data['textcolor']
        color = _get_color(color, 'TEXT')
        return color

    def clean_bordercolor(self):
        color = self.cleaned_data['bordercolor']
        color = _get_color(color, 'BORDER')
        return color

    def clean(self):
        cleaned_data = super(DummyImageForm, self).clean()
        text = cleaned_data.get('text')
        width = cleaned_data.get('width')
        height = cleaned_data.get('height')

        if not text and width and height:
            text = '%d x %d' % (width, height)

        return cleaned_data



#!/usr/bin/env python
from setuptools import setup, find_packages

setup(
    name = "django-dummyimage",
    version = "0.1.1",
    description = "Dynamic Dummy Image Generator For Django!",
    author = "Rolando Espinoza La fuente",
    author_email = "darkrho@gmail.com",
    url = "https://github.com/darkrho/django-dummyimage",
    license = "BSD",
    packages = find_packages(),
    zip_safe=False, # because we're including media that Django needs
    include_package_data = True,
    install_requires = [
        'django',
        'pil',
    ],
    classifiers=[
        'Programming Language :: Python',
        'Framework :: Django',
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
    ],
)
